@@ -32,8 +32,6 @@
 #include "eel-lib-self-check-functions.h"
 #include "eel-string.h"
 #include "eel-marshal.h"
-#include "eel-types.h"
-#include "eel-type-builtins.h"
 #include <gtk/gtk.h>
 #include <eel/eel-canvas.h>
 #include <eel/eel-canvas-util.h>
@@ -110,8 +108,6 @@ eel_background_class_init (gpointer klass)
 
 	object_class = G_OBJECT_CLASS (klass);
 
-	eel_type_init ();
-
 	signals[APPEARANCE_CHANGED] =
 		g_signal_new ("appearance_changed",
 			      G_TYPE_FROM_CLASS (object_class),
@@ -370,8 +366,6 @@ eel_background_ensure_realized (EelBackground *background, GdkWindow *window)
 		} else {
 			background->details->default_color = style->bg[GTK_STATE_NORMAL];
 		}
-		
-		gdk_rgb_find_color (style->colormap, &(background->details->default_color));
 	}
 
 	/* If the window size is the same as last time, don't update */
@@ -466,45 +460,40 @@ eel_background_expose (GtkWidget                   *widget,
 	int window_width;
 	int window_height;
 	GdkPixmap *pixmap;
-	GdkGC *gc;
-	GdkGCValues gc_values;
-	GdkGCValuesMask value_mask;
+	GdkWindow *widget_window;
+	cairo_t *cr;
 
 	EelBackground *background;
-	
-	if (event->window != widget->window) {
+
+	widget_window = gtk_widget_get_window (widget);
+	if (event->window != widget_window) {
 		return;
 	}
 
 	background = eel_get_widget_background (widget);
 
-	drawable_get_adjusted_size (background, widget->window, &window_width, &window_height);
+	drawable_get_adjusted_size (background, widget_window, &window_width, &window_height);
 	
 	pixmap = eel_background_get_pixmap_and_color (background,
-						      widget->window,
+						      widget_window,
 						      &color);
+	cr = gdk_cairo_create (widget_window);
 
 	if (pixmap) {
-		gc_values.tile = pixmap;
-		gc_values.ts_x_origin = 0;
-		gc_values.ts_y_origin = 0;
-		gc_values.fill = GDK_TILED;
-		value_mask = GDK_GC_FILL | GDK_GC_TILE | GDK_GC_TS_X_ORIGIN | GDK_GC_TS_Y_ORIGIN;
+		gdk_cairo_set_source_pixmap (cr, pixmap, 0, 0);
+		cairo_pattern_set_extend (cairo_get_source (cr), CAIRO_EXTEND_REPEAT);
 	} else {
-		gdk_rgb_find_color (gtk_widget_get_colormap (widget), &color);
-		gc_values.foreground = color;
-		gc_values.fill = GDK_SOLID;
-		value_mask = GDK_GC_FILL | GDK_GC_FOREGROUND;
+		gdk_cairo_set_source_color (cr, &color);
 	}
-	
-	gc = gdk_gc_new_with_values (widget->window, &gc_values, value_mask);
-	
-	gdk_gc_set_clip_rectangle (gc, &event->area);
 
-	gdk_draw_rectangle (widget->window, gc, TRUE, 0, 0, window_width, window_height);
-	
-	g_object_unref (gc);
-	
+	gdk_cairo_rectangle (cr, &event->area);
+	cairo_clip (cr);
+
+	cairo_rectangle (cr, 0, 0, window_width, window_height);
+	cairo_fill (cr);
+
+	cairo_destroy (cr);
+
 	if (pixmap) {
 		g_object_unref (pixmap);
 	}
@@ -748,26 +737,26 @@ eel_background_set_up_widget (EelBackground *background, GtkWidget *widget)
 	int window_height;
 	
 	GdkWindow *window;
+	GdkWindow *widget_window;
 	gboolean in_fade;
 
-	if (!GTK_WIDGET_REALIZED (widget)) {
+	if (!gtk_widget_get_realized (widget)) {
 		return;
 	}
 
-	drawable_get_adjusted_size (background, widget->window, &window_width, &window_height);
+	widget_window = gtk_widget_get_window (widget);
+	drawable_get_adjusted_size (background, widget_window, &window_width, &window_height);
 	
 	pixmap = eel_background_get_pixmap_and_color (background,
-						      widget->window,
+						      widget_window,
 						      &color);
 
 	style = gtk_widget_get_style (widget);
 	
-	gdk_rgb_find_color (style->colormap, &color);
-
 	if (EEL_IS_CANVAS (widget)) {
-		window = GTK_LAYOUT (widget)->bin_window;
+		window = gtk_layout_get_bin_window (GTK_LAYOUT (widget));
 	} else {
-		window = widget->window;
+		window = widget_window;
 	}
 
 	if (background->details->fade != NULL) {
@@ -814,7 +803,7 @@ on_background_changed (EelBackground *background)
 static void
 init_fade (EelBackground *background, GtkWidget *widget)
 {
-	if (widget == NULL || !GTK_WIDGET_REALIZED (widget))
+	if (widget == NULL || !gtk_widget_get_realized (widget))
 		return;
 
 	if (!background->details->is_desktop) {
@@ -822,13 +811,15 @@ init_fade (EelBackground *background, GtkWidget *widget)
 	}
 
 	if (background->details->fade == NULL) {
+		GdkWindow *window;
 		int old_width, old_height, width, height;
 
 		/* If this was the result of a screen size change,
 		 * we don't want to crossfade
 		 */
-		gdk_drawable_get_size (widget->window, &old_width, &old_height);
-		drawable_get_adjusted_size (background, widget->window,
+		window = gtk_widget_get_window (widget);
+		gdk_drawable_get_size (window, &old_width, &old_height);
+		drawable_get_adjusted_size (background, window,
 					    &width, &height);
 		if (old_width == width && old_height == height) {
 			background->details->fade = gnome_bg_crossfade_new (width, height);
@@ -966,7 +957,7 @@ eel_background_set_desktop (EelBackground *background, GtkWidget *widget, gboole
 {
 	background->details->is_desktop = is_desktop;
 
-	if (GTK_WIDGET_REALIZED(widget) && background->details->is_desktop) {
+	if (gtk_widget_get_realized (widget) && background->details->is_desktop) {
 		widget_realized_setup (widget, background);
 	}
 	
@@ -1075,17 +1066,19 @@ eel_background_receive_dropped_color (EelBackground *background,
 	char *color_spec;
 	char *new_gradient_spec;
 	int left_border, right_border, top_border, bottom_border;
+	GtkAllocation allocation;
 
 	g_return_if_fail (EEL_IS_BACKGROUND (background));
 	g_return_if_fail (GTK_IS_WIDGET (widget));
 	g_return_if_fail (selection_data != NULL);
 
 	/* Convert the selection data into a color spec. */
-	if (selection_data->length != 8 || selection_data->format != 16) {
+	if (gtk_selection_data_get_length ((GtkSelectionData *) selection_data) != 8 ||
+	    gtk_selection_data_get_format ((GtkSelectionData *) selection_data) != 16) {
 		g_warning ("received invalid color data");
 		return;
 	}
-	channels = (guint16 *) selection_data->data;
+	channels = (guint16 *) gtk_selection_data_get_data ((GtkSelectionData *) selection_data);
 	color_spec = g_strdup_printf ("#%02X%02X%02X",
 				      channels[0] >> 8,
 				      channels[1] >> 8,
@@ -1095,10 +1088,11 @@ eel_background_receive_dropped_color (EelBackground *background,
 	   For the moment, this is hard-wired, but later the widget will have to have some
 	   say in where the borders are.
 	*/
+	gtk_widget_get_allocation (widget, &allocation);
 	left_border = 32;
-	right_border = widget->allocation.width - 32;
+	right_border = allocation.width - 32;
 	top_border = 32;
-	bottom_border = widget->allocation.height - 32;
+	bottom_border = allocation.height - 32;
 	if (drop_location_x < left_border && drop_location_x <= right_border) {
 		new_gradient_spec = eel_gradient_set_left_color_spec (background->details->color, color_spec);
 	} else if (drop_location_x >= left_border && drop_location_x > right_border) {
