@@ -70,10 +70,7 @@ static const GtkTargetEntry drop_types [] = {
 	/* prefer "_NETSCAPE_URL" over "text/uri-list" to satisfy web browsers. */
 	{ NAUTILUS_ICON_DND_NETSCAPE_URL_TYPE, 0, NAUTILUS_ICON_DND_NETSCAPE_URL },
 	{ NAUTILUS_ICON_DND_URI_LIST_TYPE, 0, NAUTILUS_ICON_DND_URI_LIST },
-	{ NAUTILUS_ICON_DND_COLOR_TYPE, 0, NAUTILUS_ICON_DND_COLOR },
 	{ NAUTILUS_ICON_DND_BGIMAGE_TYPE, 0, NAUTILUS_ICON_DND_BGIMAGE },
-	{ NAUTILUS_ICON_DND_KEYWORD_TYPE, 0, NAUTILUS_ICON_DND_KEYWORD },
-	{ NAUTILUS_ICON_DND_RESET_BACKGROUND_TYPE,  0, NAUTILUS_ICON_DND_RESET_BACKGROUND },
 	{ NAUTILUS_ICON_DND_XDNDDIRECTSAVE_TYPE, 0, NAUTILUS_ICON_DND_XDNDDIRECTSAVE }, /* XDS Protocol Type */
 	{ NAUTILUS_ICON_DND_RAW_TYPE, 0, NAUTILUS_ICON_DND_RAW },
 	/* Must be last: */
@@ -96,7 +93,6 @@ create_selection_shadow (NautilusIconContainer *container,
 {
 	EelCanvasGroup *group;
 	EelCanvas *canvas;
-	GdkBitmap *stipple;
 	int max_x, max_y;
 	int min_x, min_y;
 	GList *p;
@@ -111,9 +107,6 @@ create_selection_shadow (NautilusIconContainer *container,
 		return NULL;
 	}
 		
-	stipple = container->details->dnd_info->stipple;
-	g_return_val_if_fail (stipple != NULL, NULL);
-
 	canvas = EEL_CANVAS (container);
 	gtk_widget_get_allocation (GTK_WIDGET (container), &allocation);
 
@@ -158,7 +151,7 @@ create_selection_shadow (NautilusIconContainer *container,
 				 "x2", (double) x2,
 				 "y2", (double) y2,
 				 "outline_color", "black",
-				 "outline_stipple", stipple,
+				 "outline-stippling", TRUE,
 				 "width_pixels", 1,
 				 NULL);
 	}
@@ -626,29 +619,6 @@ get_background_drag_action (NautilusIconContainer *container,
 	return action;
 }
 
-static void
-receive_dropped_color (NautilusIconContainer *container,
-		       int x, int y,
-		       GdkDragAction action,
-		       GtkSelectionData *data)
-{
-	action = get_background_drag_action (container, action);
-
-	if (action > 0) {
-		char *uri;
-
-		uri = get_container_uri (container);
-		nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
-				    "dropped color on icon container displaying %s", uri);
-		g_free (uri);
-
-		eel_background_receive_dropped_color
-			(eel_get_widget_background (GTK_WIDGET (container)),
-			 GTK_WIDGET (container), 
-			 action, x, y, data);
-	}
-}
-
 /* handle dropped tile images */
 static void
 receive_dropped_tile_image (NautilusIconContainer *container, GdkDragAction action, GtkSelectionData *data)
@@ -672,47 +642,6 @@ receive_dropped_tile_image (NautilusIconContainer *container, GdkDragAction acti
 	}
 }
 
-/* handle dropped keywords */
-static void
-receive_dropped_keyword (NautilusIconContainer *container, const char *keyword, int x, int y)
-{
-	char *uri;
-	double world_x, world_y;
-
-	NautilusIcon *drop_target_icon;
-	NautilusFile *file;
-	
-	g_assert (keyword != NULL);
-
-	/* find the item we hit with our drop, if any */
-	canvas_widget_to_world (EEL_CANVAS (container), x, y, &world_x, &world_y);
-	drop_target_icon = nautilus_icon_container_item_at (container, world_x, world_y);
-	if (drop_target_icon == NULL) {
-		return;
-	}
-
-	/* FIXME bugzilla.gnome.org 42485: 
-	 * This does not belong in the icon code.
-	 * It has to be in the file manager.
-	 * The icon code has no right to deal with the file directly.
-	 * But luckily there's no issue of not getting a file object,
-	 * so we don't have to worry about async. issues here.
-	 */
-	uri = nautilus_icon_container_get_icon_uri (container, drop_target_icon);
-
-	nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
-			    "dropped emblem '%s' on icon container URI: %s",
-			    keyword, uri);
-
-	file = nautilus_file_get_by_uri (uri);
-	g_free (uri);
-	
-	nautilus_drag_file_receive_dropped_keyword (file, keyword);
-
-	nautilus_file_unref (file);
-	nautilus_icon_container_update_icon (container, drop_target_icon);
-}
-
 /* handle dropped url */
 static void
 receive_dropped_netscape_url (NautilusIconContainer *container, const char *encoded_url, GdkDragContext *context, int x, int y)
@@ -1316,20 +1245,11 @@ nautilus_icon_container_get_drop_action (NautilusIconContainer *container,
 		g_free (drop_target);
 		break;
 
-	/* handle emblems by setting the action if we're over an object */
-	case NAUTILUS_ICON_DND_KEYWORD:
-		if (icon != NULL) {
-			*action = gdk_drag_context_get_suggested_action (context);
-		}
-		break;
-
 	case NAUTILUS_ICON_DND_NETSCAPE_URL:
 		*action = nautilus_drag_default_drop_action_for_netscape_url (context);
 		break;
 
-	case NAUTILUS_ICON_DND_COLOR:
 	case NAUTILUS_ICON_DND_BGIMAGE:
-	case NAUTILUS_ICON_DND_RESET_BACKGROUND:
 	case NAUTILUS_ICON_DND_ROOTWINDOW_DROP:
 		*action = gdk_drag_context_get_suggested_action (context);
 		break;
@@ -1386,7 +1306,7 @@ nautilus_icon_dnd_update_drop_target (NautilusIconContainer *container,
 	 */
 
 	/* Find if target icon accepts our drop. */
-	if (icon != NULL && (container->details->dnd_info->drag_info.data_type != NAUTILUS_ICON_DND_KEYWORD)) {
+	if (icon != NULL) {
 		    uri = nautilus_icon_container_get_icon_uri (container, icon);
 		    file = nautilus_file_get_by_uri (uri);
 		    g_free (uri);
@@ -1553,6 +1473,7 @@ drag_highlight_expose (GtkWidget      *widget,
 {
 	gint x, y, width, height;
 	GdkWindow *window;
+	cairo_t *cr;
 	
 	x = gtk_adjustment_get_value (gtk_layout_get_hadjustment (GTK_LAYOUT (widget)));
 	y = gtk_adjustment_get_value (gtk_layout_get_vadjustment (GTK_LAYOUT (widget)));
@@ -1565,11 +1486,13 @@ drag_highlight_expose (GtkWidget      *widget,
 			  NULL, widget, "dnd",
 			  x, y, width, height);
   
-	gdk_draw_rectangle (window,
-			    (gtk_widget_get_style(widget))->black_gc,
-			    FALSE,
-			    x, y, width - 1, height - 1);
-	
+	cr = gdk_cairo_create (window);
+	cairo_set_line_width (cr, 1.0);
+	cairo_set_source_rgb (cr, 0, 0, 0);
+	cairo_rectangle (cr, x + 0.5, y + 0.5, width - 1, height - 1);
+	cairo_stroke (cr);
+	cairo_destroy (cr);
+
 	return FALSE;
 }
 
@@ -1732,7 +1655,6 @@ drag_data_received_callback (GtkWidget *widget,
 			     gpointer user_data)
 {
     	NautilusDragInfo *drag_info;
-	EelBackground *background;
 	char *tmp;
 	const char *tmp_raw;
 	int length;
@@ -1747,12 +1669,9 @@ drag_data_received_callback (GtkWidget *widget,
 	case NAUTILUS_ICON_DND_GNOME_ICON_LIST:
 		nautilus_icon_container_dropped_icon_feedback (widget, data, x, y);
 		break;
-	case NAUTILUS_ICON_DND_COLOR:
 	case NAUTILUS_ICON_DND_BGIMAGE:	
-	case NAUTILUS_ICON_DND_KEYWORD:
 	case NAUTILUS_ICON_DND_URI_LIST:
 	case NAUTILUS_ICON_DND_TEXT:
-	case NAUTILUS_ICON_DND_RESET_BACKGROUND:
 	case NAUTILUS_ICON_DND_XDNDDIRECTSAVE:
 	case NAUTILUS_ICON_DND_RAW:
 		/* Save the data so we can do the actual work on drop. */
@@ -1786,24 +1705,12 @@ drag_data_received_callback (GtkWidget *widget,
 				(NAUTILUS_ICON_CONTAINER (widget),
 				 context, x, y);
 			break;
-		case NAUTILUS_ICON_DND_COLOR:
-			receive_dropped_color (NAUTILUS_ICON_CONTAINER (widget),
-					       x, y,
-					       gdk_drag_context_get_selected_action (context),
-					       data);
-			success = TRUE;
-			break;
 		case NAUTILUS_ICON_DND_BGIMAGE:
 			receive_dropped_tile_image
 				(NAUTILUS_ICON_CONTAINER (widget),
 				 gdk_drag_context_get_selected_action (context),
 				 data);
 			break;
-		case NAUTILUS_ICON_DND_KEYWORD:
-			receive_dropped_keyword
-				(NAUTILUS_ICON_CONTAINER (widget),
-				 (char *) gtk_selection_data_get_data (data), x, y);
-			break;
 		case NAUTILUS_ICON_DND_NETSCAPE_URL:
 			receive_dropped_netscape_url
 				(NAUTILUS_ICON_CONTAINER (widget),
@@ -1833,13 +1740,6 @@ drag_data_received_callback (GtkWidget *widget,
 				 context, x, y);
 			success = TRUE;
 			break;
-		case NAUTILUS_ICON_DND_RESET_BACKGROUND:
-			background = eel_get_widget_background (widget);
-			if (background != NULL) {
-				eel_background_reset (background);
-			}
-			gtk_drag_finish (context, FALSE, FALSE, time);			
-			break;
 		case NAUTILUS_ICON_DND_ROOTWINDOW_DROP:
 			/* Do nothing, everything is done by the sender */
 			break;
@@ -1897,23 +1797,7 @@ drag_data_received_callback (GtkWidget *widget,
 }
 
 void
-nautilus_icon_dnd_set_stipple (NautilusIconContainer *container,
-			       GdkBitmap             *stipple)
-{
-	if (stipple != NULL) {
-		g_object_ref (stipple);
-	}
-	
-	if (container->details->dnd_info->stipple != NULL) {
-		g_object_unref (container->details->dnd_info->stipple);
-	}
-
-	container->details->dnd_info->stipple = stipple;
-}
-
-void
-nautilus_icon_dnd_init (NautilusIconContainer *container,
-			GdkBitmap *stipple)
+nautilus_icon_dnd_init (NautilusIconContainer *container)
 {
 	GtkTargetList *targets;
 	int n_elements;
@@ -1961,10 +1845,6 @@ nautilus_icon_dnd_init (NautilusIconContainer *container,
 			  G_CALLBACK (drag_drop_callback), NULL);
 	g_signal_connect (container, "drag_leave",
 			  G_CALLBACK (drag_leave_callback), NULL);
-
-	if (stipple != NULL) {
-		container->details->dnd_info->stipple = g_object_ref (stipple);
-	}
 }
 
 void
@@ -1975,10 +1855,6 @@ nautilus_icon_dnd_fini (NautilusIconContainer *container)
 	if (container->details->dnd_info != NULL) {
 		stop_auto_scroll (container);
 
-		if (container->details->dnd_info->stipple != NULL) {
-			g_object_unref (container->details->dnd_info->stipple);
-		}
-
 		nautilus_drag_finalize (&container->details->dnd_info->drag_info);
 		container->details->dnd_info = NULL;
 	}
