@@ -26,6 +26,8 @@
 #include <config.h>
 #include "nautilus-thumbnails.h"
 
+#define GNOME_DESKTOP_USE_UNSTABLE_API
+
 #include "nautilus-directory-notify.h"
 #include "nautilus-global-preferences.h"
 #include "nautilus-file-utilities.h"
@@ -33,9 +35,9 @@
 #include <eel/eel-gdk-pixbuf-extensions.h>
 #include <eel/eel-graphic-effects.h>
 #include <eel/eel-string.h>
+#include <eel/eel-debug.h>
 #include <eel/eel-vfs-extensions.h>
-#include <gtk/gtkmain.h>
-#include <librsvg/rsvg.h>
+#include <gtk/gtk.h>
 #include <errno.h>
 #include <stdio.h>
 #include <string.h>
@@ -43,7 +45,7 @@
 #include <sys/wait.h>
 #include <unistd.h>
 #include <signal.h>
-#include <libgnomeui/gnome-thumbnail.h>
+#include <libgnomeui/gnome-desktop-thumbnail.h>
 
 #include "nautilus-file-private.h"
 
@@ -112,15 +114,7 @@ static GHashTable *thumbnails_to_make_hash = NULL;
  * to avoid adding it again. Lock thumbnails_mutex when accessing this. */
 static NautilusThumbnailInfo *currently_thumbnailing = NULL;
 
-static GnomeThumbnailFactory *thumbnail_factory = NULL;
-
-static const char *types [] = {
-	"image/x-bmp", "image/x-ico", "image/jpeg", "image/gif",
-	"image/png", "image/pnm", "image/ras", "image/tga",
-	"image/tiff", "image/wbmp", "image/bmp", "image/x-xbitmap",
-	"image/x-xpixmap"
-};
-static GHashTable *image_mime_types = NULL;
+static GnomeDesktopThumbnailFactory *thumbnail_factory = NULL;
 
 static int thumbnail_icon_size = 0;
 
@@ -129,18 +123,24 @@ get_file_mtime (const char *file_uri, time_t* mtime)
 {
 	GFile *file;
 	GFileInfo *info;
+	gboolean ret;
 
+	ret = FALSE;
 	*mtime = INVALID_MTIME;
 
 	file = g_file_new_for_uri (file_uri);
 	info = g_file_query_info (file, G_FILE_ATTRIBUTE_TIME_MODIFIED, 0, NULL, NULL);
 	if (info) {
-		*mtime = g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_TIME_MODIFIED);
+		if (g_file_info_has_attribute (info, G_FILE_ATTRIBUTE_TIME_MODIFIED)) {
+			*mtime = g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_TIME_MODIFIED);
+			ret = TRUE;
+		}
+
 		g_object_unref (info);
 	}
 	g_object_unref (file);
-	
-	return TRUE;
+
+	return ret;
 }
 
 static void
@@ -151,13 +151,13 @@ free_thumbnail_info (NautilusThumbnailInfo *info)
 	g_free (info);
 }
 
-static GnomeThumbnailFactory *
+static GnomeDesktopThumbnailFactory *
 get_thumbnail_factory (void)
 {
-	static GnomeThumbnailFactory *thumbnail_factory = NULL;
+	static GnomeDesktopThumbnailFactory *thumbnail_factory = NULL;
 
 	if (thumbnail_factory == NULL) {
-		thumbnail_factory = gnome_thumbnail_factory_new (GNOME_THUMBNAIL_SIZE_NORMAL);
+		thumbnail_factory = gnome_desktop_thumbnail_factory_new (GNOME_DESKTOP_THUMBNAIL_SIZE_NORMAL);
 	}
 
 	return thumbnail_factory;
@@ -209,32 +209,27 @@ nautilus_update_thumbnail_file_copied (const char *source_file_uri,
 				       const char *destination_file_uri)
 {
 	char *old_thumbnail_path;
+	time_t mtime;
 	GdkPixbuf *pixbuf;
-	GFileInfo *file_info;
-	GnomeThumbnailFactory *factory;
-	GFile *destination_file;
+	GnomeDesktopThumbnailFactory *factory;
 	
-	old_thumbnail_path = gnome_thumbnail_path_for_uri (source_file_uri, GNOME_THUMBNAIL_SIZE_NORMAL);
+	old_thumbnail_path = gnome_desktop_thumbnail_path_for_uri (source_file_uri, GNOME_DESKTOP_THUMBNAIL_SIZE_NORMAL);
 	if (old_thumbnail_path != NULL &&
 	    g_file_test (old_thumbnail_path, G_FILE_TEST_EXISTS)) {
-		destination_file = g_file_new_for_uri (destination_file_uri);
-		file_info = g_file_query_info (destination_file, G_FILE_ATTRIBUTE_TIME_MODIFIED, 0, NULL, NULL);
-		g_object_unref (destination_file);
-		if (file_info != NULL) {
+		if (get_file_mtime (destination_file_uri, &mtime)) {
 			pixbuf = gdk_pixbuf_new_from_file (old_thumbnail_path, NULL);
 			
-			if (pixbuf && gnome_thumbnail_has_uri (pixbuf, source_file_uri)) {
+			if (pixbuf && gnome_desktop_thumbnail_has_uri (pixbuf, source_file_uri)) {
 				factory = get_thumbnail_factory ();
-				gnome_thumbnail_factory_save_thumbnail (factory,
-									pixbuf,
-									destination_file_uri,
-									g_file_info_get_attribute_uint64 (file_info, G_FILE_ATTRIBUTE_TIME_MODIFIED));
+				gnome_desktop_thumbnail_factory_save_thumbnail (factory,
+										pixbuf,
+										destination_file_uri,
+										mtime);
 			}
 			
 			if (pixbuf) {
 				g_object_unref (pixbuf);
 			}
-			g_object_unref (file_info);
 		}
 	}
 
@@ -254,7 +249,7 @@ nautilus_remove_thumbnail_for_file (const char *file_uri)
 {
 	char *thumbnail_path;
 	
-	thumbnail_path = gnome_thumbnail_path_for_uri (file_uri, GNOME_THUMBNAIL_SIZE_NORMAL);
+	thumbnail_path = gnome_desktop_thumbnail_path_for_uri (file_uri, GNOME_DESKTOP_THUMBNAIL_SIZE_NORMAL);
 	if (thumbnail_path != NULL) {
 		unlink (thumbnail_path);
 	}
@@ -430,8 +425,9 @@ get_pixbuf_from_data (const unsigned char *buffer,
 	GError *error;
 
 	if (thumbnail_icon_size == 0) {
-		eel_preferences_add_auto_integer (NAUTILUS_PREFERENCES_ICON_VIEW_THUMBNAIL_SIZE,
-						  &thumbnail_icon_size);
+		eel_g_settings_add_auto_int (nautilus_icon_view_preferences,
+					     NAUTILUS_PREFERENCES_ICON_VIEW_THUMBNAIL_SIZE,
+					     &thumbnail_icon_size);
 	}
 
 	loader = gdk_pixbuf_loader_new ();
@@ -466,7 +462,12 @@ get_pixbuf_from_data (const unsigned char *buffer,
 	if (!gdk_pixbuf_loader_close (loader, &error) ||
 	    /* Seems we have to check this even if it returned TRUE (#403255) */
 	    error != NULL) {
-		g_message ("Failed to close thumbnail pixbuf loader for %s: %s", path, error->message);
+		/* In some cases, we don't get an error even with FALSE returns (#538888) */
+		if (error != NULL) {
+			g_message ("Failed to close thumbnail pixbuf loader for %s: %s", path, error->message);
+		} else {
+			g_message ("Failed to close thumbnail pixbuf loader for %s", path);
+		}
 
 		g_object_unref (G_OBJECT (loader));
 		g_error_free (error);
@@ -548,7 +549,7 @@ async_thumbnail_read_image (GObject *source_object,
 			   pixbuf, scale_x, scale_y,
 			   handle->load_func_user_data);
 
-	gdk_pixbuf_unref (pixbuf);
+	g_object_unref (pixbuf);
 
 	g_object_unref (handle->cancellable);
 	g_free (handle->file_path);
@@ -739,37 +740,72 @@ thumbnail_thread_notify_file_changed (gpointer image_uri)
 static GHashTable *
 get_types_table (void)
 {
+	static GHashTable *image_mime_types = NULL;
+	GSList *format_list, *l;
+	char **types;
 	int i;
 
 	if (image_mime_types == NULL) {
-		image_mime_types = g_hash_table_new (g_str_hash, g_str_equal);
-		for (i = 0; i < G_N_ELEMENTS (types); i++) {
-			g_hash_table_insert (image_mime_types,
-					     (gpointer) types [i],
-					     GUINT_TO_POINTER (1));
+		image_mime_types =
+			g_hash_table_new_full (g_str_hash, g_str_equal,
+					       g_free, NULL);
+		eel_debug_call_at_shutdown_with_data ((GFreeFunc)g_hash_table_destroy,
+						      image_mime_types);
+
+		format_list = gdk_pixbuf_get_formats ();
+		for (l = format_list; l; l = l->next) {
+			types = gdk_pixbuf_format_get_mime_types (l->data);
+
+			for (i = 0; i < G_N_ELEMENTS (types); i++) {
+				g_hash_table_insert (image_mime_types,
+						     types [i],
+						     GUINT_TO_POINTER (1));
+			}
+
+			g_free (types);
 		}
+
+		g_slist_free (format_list);
 	}
 
 	return image_mime_types;
 }
 
+static gboolean
+pixbuf_can_load_type (const char *mime_type)
+{
+	GHashTable *image_mime_types;
+
+	image_mime_types = get_types_table ();
+	if (g_hash_table_lookup (image_mime_types, mime_type)) {
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+gboolean
+nautilus_can_thumbnail_internally (NautilusFile *file)
+{
+	char *mime_type;
+	gboolean res;
+
+	mime_type = nautilus_file_get_mime_type (file);
+	res = pixbuf_can_load_type (mime_type);
+	g_free (mime_type);
+	return res;
+}
+
 gboolean
 nautilus_thumbnail_is_mimetype_limited_by_size (const char *mime_type)
 {
-	GHashTable *image_mime_types;
-	
-	image_mime_types = get_types_table ();
-        if (g_hash_table_lookup (image_mime_types, mime_type)) {
-                return TRUE;
-	}
-
-        return FALSE;
+	return pixbuf_can_load_type (mime_type);
 }
 
 gboolean
 nautilus_can_thumbnail (NautilusFile *file)
 {
-	GnomeThumbnailFactory *factory;
+	GnomeDesktopThumbnailFactory *factory;
 	gboolean res;
 	char *uri;
 	time_t mtime;
@@ -780,32 +816,16 @@ nautilus_can_thumbnail (NautilusFile *file)
 	mtime = nautilus_file_get_mtime (file);
 	
 	factory = get_thumbnail_factory ();
-	res = gnome_thumbnail_factory_can_thumbnail (factory,
-						     uri,
-						     mime_type,
-						     mtime);
+	res = gnome_desktop_thumbnail_factory_can_thumbnail (factory,
+							     uri,
+							     mime_type,
+							     mtime);
 	g_free (mime_type);
 	g_free (uri);
 
 	return res;
 }
 
-gboolean
-nautilus_has_valid_failed_thumbnail (NautilusFile *file)
-{
-	GnomeThumbnailFactory *factory;
-	char *uri;
-	gboolean res;
-
-	factory = get_thumbnail_factory ();
-
-	uri = nautilus_file_get_uri (file);
-	res = gnome_thumbnail_factory_has_valid_failed_thumbnail (factory, uri, file->details->mtime);
-	g_free (uri);
-
-	return res;
-}
-
 void
 nautilus_create_thumbnail (NautilusFile *file)
 {
@@ -967,7 +987,7 @@ thumbnail_thread_start (gpointer data)
 				   info->image_uri);
 #endif
 			/* Reschedule thumbnailing via a change notification */
-			g_timeout_add (1000, thumbnail_thread_notify_file_changed,
+			g_timeout_add_seconds (1, thumbnail_thread_notify_file_changed,
 				       g_strdup (info->image_uri));
  			continue;
 		}
@@ -978,20 +998,20 @@ thumbnail_thread_start (gpointer data)
 			   info->image_uri);
 #endif
 
-		pixbuf = gnome_thumbnail_factory_generate_thumbnail (thumbnail_factory,
-								     info->image_uri,
-								     info->mime_type);
+		pixbuf = gnome_desktop_thumbnail_factory_generate_thumbnail (thumbnail_factory,
+									     info->image_uri,
+									     info->mime_type);
 
 		if (pixbuf) {
-			gnome_thumbnail_factory_save_thumbnail (thumbnail_factory,
-								pixbuf,
-								info->image_uri,
-								current_orig_mtime);
+			gnome_desktop_thumbnail_factory_save_thumbnail (thumbnail_factory,
+									pixbuf,
+									info->image_uri,
+									current_orig_mtime);
 			g_object_unref (pixbuf);
 		} else {
-			gnome_thumbnail_factory_create_failed_thumbnail (thumbnail_factory, 
-									 info->image_uri,
-									 current_orig_mtime);
+			gnome_desktop_thumbnail_factory_create_failed_thumbnail (thumbnail_factory, 
+										 info->image_uri,
+										 current_orig_mtime);
 		}
 		/* We need to call nautilus_file_changed(), but I don't think that is
 		   thread safe. So add an idle handler and do it from the main loop. */
