@@ -35,11 +35,9 @@
 #if ENABLE_EMPTY_VIEW
 #include "file-manager/fm-empty-view.h"
 #endif /* ENABLE_EMPTY_VIEW */
-#include "nautilus-information-panel.h"
 #include "nautilus-history-sidebar.h"
 #include "nautilus-places-sidebar.h"
 #include "nautilus-notes-viewer.h"
-#include "nautilus-emblem-sidebar.h"
 #include "nautilus-image-properties-page.h"
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -56,6 +54,7 @@
 #include "nautilus-window-private.h"
 #include "nautilus-window-manage-views.h"
 #include <unistd.h>
+#include <errno.h>
 #include <libxml/xmlsave.h>
 #include <glib/gstdio.h>
 #include <glib/gi18n.h>
@@ -76,14 +75,10 @@
 #include <libnautilus-extension/nautilus-menu-provider.h>
 #include <libnautilus-private/nautilus-autorun.h>
 
-enum
-{
-  COMMAND_0, /* unused: 0 is an invalid command */
-
-  COMMAND_START_DESKTOP,
-  COMMAND_STOP_DESKTOP,
-  COMMAND_OPEN_BROWSER,
-};
+#define COMMAND_OPEN "open"
+#define COMMAND_START_DESKTOP "start_desktop"
+#define COMMAND_STOP_DESKTOP "stop_desktop"
+#define COMMAND_CLOSE "close"
 
 /* Keep window from shrinking down ridiculously small; numbers are somewhat arbitrary */
 #define APPLICATION_WINDOW_MIN_WIDTH	300
@@ -126,73 +121,6 @@ static char *   nautilus_application_get_session_data (void);
 
 G_DEFINE_TYPE (NautilusApplication, nautilus_application, G_TYPE_OBJECT);
 
-static gboolean
-_unique_message_data_set_geometry_and_uris (UniqueMessageData  *message_data,
-					    const char *geometry,
-					    char **uris)
-{
-  GString *list;
-  gint i;
-  gchar *result;
-  gsize length;
-
-  list = g_string_new (NULL);
-  if (geometry != NULL) {
-	  g_string_append (list, geometry);
-  }
-  g_string_append (list, "\r\n");
-  
-  for (i = 0; uris != NULL && uris[i]; i++) {
-	  g_string_append (list, uris[i]);
-	  g_string_append (list, "\r\n");
-  }
-
-  result = g_convert (list->str, list->len,
-                      "ASCII", "UTF-8",
-                      NULL, &length, NULL);
-  g_string_free (list, TRUE);
-  
-  if (result) {
-	  unique_message_data_set (message_data, (guchar *) result, length);
-	  g_free (result);
-	  return TRUE;
-  }
-  
-  return FALSE;
-}
-
-static gchar **
-_unique_message_data_get_geometry_and_uris (UniqueMessageData *message_data,
-					    char **geometry)
-{
-  gchar **result = NULL;
-
-  *geometry = NULL;
-  
-  gchar *text, *newline, *uris;
-  text = unique_message_data_get_text (message_data);
-  if (text) {
-	  newline = strchr (text, '\n');
-	  if (newline) {
-		  *geometry = g_strndup (text, newline-text);
-		  uris = newline+1;
-	  } else {
-		  uris = text;
-	  }
-	  
-	  result = g_uri_list_extract_uris (uris);
-	  g_free (text);
-  }
-
-  /* if the string is empty, make it NULL */
-  if (*geometry && strlen (*geometry) == 0) {
-	  g_free (*geometry);
-	  *geometry = NULL;
-  }
-
-  return result;
-}
-
 GList *
 nautilus_application_get_window_list (void)
 {
@@ -226,8 +154,8 @@ automount_all_volumes (NautilusApplication *application)
 	GList *volumes, *l;
 	GMount *mount;
 	GVolume *volume;
-		
-	if (eel_preferences_get_boolean (NAUTILUS_PREFERENCES_MEDIA_AUTOMOUNT)) {
+
+	if (g_settings_get_boolean (nautilus_media_preferences, NAUTILUS_PREFERENCES_MEDIA_AUTOMOUNT)) {
 		/* automount all mountable volumes at start-up */
 		volumes = g_volume_monitor_get_volumes (application->volume_monitor);
 		for (l = volumes; l != NULL; l = l->next) {
@@ -282,13 +210,6 @@ nautilus_application_init (NautilusApplication *application)
 	/* Create an undo manager */
 	application->undo_manager = nautilus_undo_manager_new ();
 
-	application->unique_app = unique_app_new_with_commands ("org.gnome.Nautilus", NULL,
-								"start_desktop", COMMAND_START_DESKTOP,
-								"stop_desktop", COMMAND_STOP_DESKTOP,
-								"open_browser", COMMAND_OPEN_BROWSER,
-								NULL);
-
-	
         application->smclient = egg_sm_client_get ();
         g_signal_connect (application->smclient, "save_state",
                           G_CALLBACK (smclient_save_state_cb),
@@ -309,11 +230,9 @@ nautilus_application_init (NautilusApplication *application)
 
 	/* register sidebars */
 	nautilus_places_sidebar_register ();
-	nautilus_information_panel_register ();
 	fm_tree_view_register ();
 	nautilus_history_sidebar_register ();
 	nautilus_notes_viewer_register (); /* also property page */
-	nautilus_emblem_sidebar_register ();
 
 	/* register property pages */
 	nautilus_image_properties_page_register ();
@@ -666,8 +585,9 @@ static void
 do_upgrades_once (NautilusApplication *application,
 		  gboolean no_desktop)
 {
-	char *metafile_dir, *updated;
-	int fd;
+	char *metafile_dir, *updated, *nautilus_dir, *xdg_dir;
+	const gchar *message;
+	int fd, res;
 
 	if (!no_desktop) {
 		mark_desktop_files_trusted ();
@@ -687,6 +607,35 @@ do_upgrades_once (NautilusApplication *application,
 		g_free (updated);
 	}
 	g_free (metafile_dir);
+
+	nautilus_dir = g_build_filename (g_get_home_dir (),
+					 ".nautilus", NULL);
+	xdg_dir = nautilus_get_user_directory ();
+	if (g_file_test (nautilus_dir, G_FILE_TEST_IS_DIR)) {
+		/* test if we already attempted to migrate first */
+		updated = g_build_filename (nautilus_dir, "DEPRECATED-DIRECTORY", NULL);
+		message = _("Nautilus 3.0 deprecated this directory and tried migrating "
+			    "this configuration to ~/.config/nautilus");
+		if (!g_file_test (updated, G_FILE_TEST_EXISTS)) {
+			/* rename() works fine if the destination directory is
+			 * empty.
+			 */
+			res = g_rename (nautilus_dir, xdg_dir);
+
+			if (res == -1) {
+				fd = g_creat (updated, 0600);
+				if (fd != -1) {
+					res = write (fd, message, strlen (message));
+					close (fd);
+				}
+			}
+		}
+
+		g_free (updated);
+	}
+
+	g_free (nautilus_dir);
+	g_free (xdg_dir);
 }
 
 static void
@@ -749,7 +698,7 @@ open_window (NautilusApplication *application,
 	NautilusWindow *window;
 
 	if (browser_window ||
-	    eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ALWAYS_USE_BROWSER)) {
+	    g_settings_get_boolean (nautilus_preferences, NAUTILUS_PREFERENCES_ALWAYS_USE_BROWSER)) {
 		window = nautilus_application_create_navigation_window (application,
 									startup_id,
 									screen);
@@ -809,53 +758,65 @@ open_windows (NautilusApplication *application,
 	}
 }
 
-static UniqueResponse
-message_received_cb (UniqueApp         *unique_app,
-                     gint               command,
-                     UniqueMessageData *message,
-                     guint              time_,
-                     gpointer           user_data)
+static void
+message_received_cb (GApplication *unique_app,
+		     gchar *name,
+		     GVariant *platform_data,
+		     gpointer user_data)
 {
 	NautilusApplication *application;
-	UniqueResponse res;
-	char **uris;
-	char *geometry;
-	GdkScreen *screen;
-	
+
 	application =  user_data;
-	res = UNIQUE_RESPONSE_OK;
-	
-	switch (command) {
-	case UNIQUE_CLOSE:
-		res = UNIQUE_RESPONSE_OK;
+
+	if (g_strcmp0 (name, COMMAND_CLOSE) == 0) {
 		nautilus_main_event_loop_quit (TRUE);
-		
-		break;
-	case UNIQUE_OPEN:
-	case COMMAND_OPEN_BROWSER:
-		uris = _unique_message_data_get_geometry_and_uris (message, &geometry);
-		screen = unique_message_data_get_screen (message);
+		return;
+	}
+
+	if (g_strcmp0 (name, COMMAND_OPEN) == 0) {
+		GVariantIter iter;
+		gboolean use_browser;
+		gchar *key;
+		GVariant *value;
+		gchar *startup_id = NULL, *geometry = NULL;
+		char **uris = NULL;
+
+		g_variant_iter_init (&iter, platform_data);
+
+		while (g_variant_iter_next (&iter, "{&sv}", &key, &value)) {
+			if (g_strcmp0 (key, "urls") == 0) {
+				g_variant_get (value, "^as", &uris);
+			} else if (g_strcmp0 (key, "geometry") == 0) {
+				g_variant_get (value, "&s", &geometry);
+			} else if (g_strcmp0 (key, "use_browser") == 0) {
+				g_variant_get (value, "b", &use_browser);
+			} else if (g_strcmp0 (key, "startup_id") == 0) {
+				g_variant_get (value, "&s", &startup_id);
+			}
+
+			g_variant_unref (value);
+		}
+
 		open_windows (application,
-			      unique_message_data_get_startup_id (message),
+			      startup_id,
 			      uris,
-			      screen,
+			      gdk_screen_get_default (),
 			      geometry,
-			      command == COMMAND_OPEN_BROWSER);
+			      use_browser);
 		g_strfreev (uris);
-		g_free (geometry);
-		break;
-	case COMMAND_START_DESKTOP:
+
+		return;
+	}
+
+	if (g_strcmp0 (name, COMMAND_START_DESKTOP) == 0) {
 		nautilus_application_open_desktop (application);
-		break;
-	case COMMAND_STOP_DESKTOP:
+		return;
+	}
+
+	if (g_strcmp0 (name, COMMAND_STOP_DESKTOP) == 0) {
 		nautilus_application_close_desktop ();
-		break;
-	default:
-		res = UNIQUE_RESPONSE_PASSTHROUGH;
-		break;
+		return;
 	}
-	
-	return res;
 }
 
 gboolean 
@@ -887,6 +848,34 @@ queue_accel_map_save_callback (GtkAccelMap *object, gchar *accel_path,
 	}
 }
 
+static GVariant *
+get_startup_id (void)
+{
+	gchar *id = NULL;
+	GVariant *retval;
+	GTimeVal timeval = { 0, };
+
+	id = g_strdup (gdk_x11_display_get_startup_notification_id (gdk_display_get_default ()));
+
+	if (id == NULL) {
+		id = g_strdup (g_getenv ("DESKTOP_STARTUP_ID"));
+	}
+
+	if (id == NULL) {
+		guint32 timestamp;
+
+		g_get_current_time (&timeval);
+
+		timestamp =  timeval.tv_sec;
+		id = g_strdup_printf ("_TIME%lu", (unsigned long) timestamp);
+        }
+
+	retval = g_variant_new ("s", id);
+	g_free (id);
+
+	return retval;
+}
+
 void
 nautilus_application_startup (NautilusApplication *application,
 			      gboolean kill_shell,
@@ -896,8 +885,9 @@ nautilus_application_startup (NautilusApplication *application,
 			      const char *geometry,
 			      char **urls)
 {
-	UniqueMessageData *message;
-	
+	gboolean is_remote;
+	GError *error = NULL;
+
 	/* Check the user's ~/.nautilus directories and post warnings
 	 * if there are problems.
 	 */
@@ -905,60 +895,95 @@ nautilus_application_startup (NautilusApplication *application,
 		return;
 	}
 
+	application->unique_app = g_initable_new (G_TYPE_APPLICATION,
+						  NULL,
+						  &error,
+						  "application-id", "org.gnome.Nautilus",
+						  "default-quit", FALSE,
+						  "argv", g_variant_new_bytestring_array (NULL, 0),
+						  NULL);
+	g_assert (error == NULL);
+	is_remote = g_application_is_remote (application->unique_app);
+
+	if (!is_remote) {
+		/* set application properties */
+		g_application_add_action (application->unique_app,
+					  "open", "Open a nautilus window");
+		g_application_add_action (application->unique_app,
+					  "close", "Close the application");
+		g_application_add_action (application->unique_app,
+					  COMMAND_START_DESKTOP, "Starts the desktop");
+		g_application_add_action (application->unique_app,
+					  COMMAND_STOP_DESKTOP, "Stops the desktop");
+	}
+
 	if (kill_shell) {
-		if (unique_app_is_running (application->unique_app)) {
-			unique_app_send_message (application->unique_app,
-						 UNIQUE_CLOSE, NULL);
-			
+		if (is_remote) {
+			g_application_invoke_action (application->unique_app,
+						     COMMAND_CLOSE, NULL);
 		}
 	} else {
 		char *accel_map_filename;
 
 		if (!no_desktop &&
-		    !eel_preferences_get_boolean (NAUTILUS_PREFERENCES_SHOW_DESKTOP)) {
+		    !g_settings_get_boolean (nautilus_preferences, NAUTILUS_PREFERENCES_SHOW_DESKTOP)) {
 			no_desktop = TRUE;
 		}
-			
+
 		if (!no_desktop) {
-			if (unique_app_is_running (application->unique_app)) {
-				unique_app_send_message (application->unique_app,
-							 COMMAND_START_DESKTOP, NULL);
+			if (is_remote) {
+				g_application_invoke_action (application->unique_app,
+							     COMMAND_START_DESKTOP, NULL);
 			} else {
 				nautilus_application_open_desktop (application);
 			}
 		}
 
-		if (!unique_app_is_running (application->unique_app)) {
+		if (!is_remote) {
 			finish_startup (application, no_desktop);
-			g_signal_connect (application->unique_app, "message-received", G_CALLBACK (message_received_cb), application);			
+			g_signal_connect (application->unique_app,
+					  "action-with-data",
+					  G_CALLBACK (message_received_cb), application);			
 		}
-		
+
 		/* Monitor the preference to show or hide the desktop */
-		eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_SHOW_DESKTOP,
-							  desktop_changed_callback,
-							  application,
-							  G_OBJECT (application));
+		g_signal_connect_swapped (nautilus_preferences, "changed::" NAUTILUS_PREFERENCES_SHOW_DESKTOP,
+					  G_CALLBACK(desktop_changed_callback),
+					  G_OBJECT (application));
 
 		/* Monitor the preference to have the desktop */
 		/* point to the Unix home folder */
-		eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_DESKTOP_IS_HOME_DIR,
-							  desktop_location_changed_callback,
-							  NULL,
-							  G_OBJECT (application));
+		g_signal_connect_swapped (nautilus_preferences, "changed::" NAUTILUS_PREFERENCES_DESKTOP_IS_HOME_DIR,
+					  G_CALLBACK(desktop_location_changed_callback),
+					  G_OBJECT (application));
 
-	  	/* Create the other windows. */
+		/* Create the other windows. */
 		if (urls != NULL || !no_default_window) {
-			if (unique_app_is_running (application->unique_app)) {
-				message = unique_message_data_new ();
-				_unique_message_data_set_geometry_and_uris (message, geometry, urls);
-				if (browser_window) {
-					unique_app_send_message (application->unique_app,
-								 COMMAND_OPEN_BROWSER, message);
-				} else {
-					unique_app_send_message (application->unique_app,
-								 UNIQUE_OPEN, message);
+			if (is_remote) {
+				GVariant *variant;
+				GVariantBuilder builder;
+
+				g_variant_builder_init (&builder, G_VARIANT_TYPE ("a{sv}"));
+
+				if (geometry != NULL) {
+					g_variant_builder_add (&builder, "{sv}",
+							       "geometry", g_variant_new ("s", geometry));
 				}
-				unique_message_data_free (message);				
+
+				if (urls != NULL) {
+					g_variant_builder_add (&builder, "{sv}",
+							       "urls", g_variant_new ("^as", urls));
+				}
+
+				g_variant_builder_add (&builder, "{sv}",
+						       "use_browser", g_variant_new ("b", browser_window));
+				g_variant_builder_add (&builder, "{sv}",
+						       "startup_id", get_startup_id ());
+				variant = g_variant_builder_end (&builder);
+
+				g_application_invoke_action (application->unique_app,
+							     COMMAND_OPEN, variant);
+				g_variant_unref (variant);
 			} else {
 				open_windows (application, NULL,
 					      urls,
@@ -1450,19 +1475,19 @@ nautilus_application_create_navigation_window (NautilusApplication *application,
 	gboolean maximized;
 
 	g_return_val_if_fail (NAUTILUS_IS_APPLICATION (application), NULL);
-	
+
 	window = create_window (application, NAUTILUS_TYPE_NAVIGATION_WINDOW, startup_id, screen);
 
-	maximized = eel_preferences_get_boolean
-			(NAUTILUS_PREFERENCES_NAVIGATION_WINDOW_MAXIMIZED);
+	maximized = g_settings_get_boolean
+		(nautilus_window_state, NAUTILUS_WINDOW_STATE_MAXIMIZED);
 	if (maximized) {
 		gtk_window_maximize (GTK_WINDOW (window));
 	} else {
 		gtk_window_unmaximize (GTK_WINDOW (window));
 	}
 
-	geometry_string = eel_preferences_get
-			(NAUTILUS_PREFERENCES_NAVIGATION_WINDOW_SAVED_GEOMETRY);
+	geometry_string = g_settings_get_string
+		(nautilus_window_state, NAUTILUS_WINDOW_STATE_GEOMETRY);
 	if (geometry_string != NULL &&
 	    geometry_string[0] != 0) {
 		/* Ignore saved window position if a window with the same
@@ -1500,9 +1525,9 @@ static void
 desktop_changed_callback (gpointer user_data)
 {
 	NautilusApplication *application;
-	
+
 	application = NAUTILUS_APPLICATION (user_data);
-	if ( eel_preferences_get_boolean (NAUTILUS_PREFERENCES_SHOW_DESKTOP)) {
+	if (g_settings_get_boolean (nautilus_preferences, NAUTILUS_PREFERENCES_SHOW_DESKTOP)) {
 		nautilus_application_open_desktop (application);
 	} else {
 		nautilus_application_close_desktop ();
@@ -1524,7 +1549,7 @@ volume_added_callback (GVolumeMonitor *monitor,
 		       GVolume *volume,
 		       NautilusApplication *application)
 {
-	if (eel_preferences_get_boolean (NAUTILUS_PREFERENCES_MEDIA_AUTOMOUNT) &&
+	if (g_settings_get_boolean (nautilus_media_preferences, NAUTILUS_PREFERENCES_MEDIA_AUTOMOUNT) &&
 	    g_volume_should_automount (volume) &&
 	    g_volume_can_mount (volume)) {
 		nautilus_file_operations_mount_volume (NULL, volume, TRUE);
@@ -1593,11 +1618,11 @@ autorun_show_window (GMount *mount, gpointer user_data)
 {
 	GFile *location;
 	NautilusApplication *application = user_data;
-	
+
 	location = g_mount_get_root (mount);
-	
-	/* Ther should probably be an easier way to do this */
-	if (eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ALWAYS_USE_BROWSER)) {
+
+	/* There should probably be an easier way to do this */
+	if (g_settings_get_boolean (nautilus_preferences, NAUTILUS_PREFERENCES_ALWAYS_USE_BROWSER)) {
 		NautilusWindow *window;
 		window = nautilus_application_create_navigation_window (application, 
 									NULL, 
