@@ -38,7 +38,6 @@
 #include "nautilus-bookmark-list.h"
 #include "nautilus-bookmarks-window.h"
 #include "nautilus-file-management-properties.h"
-#include "nautilus-property-browser.h"
 #include "nautilus-window-manage-views.h"
 #include "nautilus-window-private.h"
 #include "nautilus-window-bookmarks.h"
@@ -47,7 +46,6 @@
 #include <eel/eel-gnome-extensions.h>
 #include <eel/eel-stock-dialogs.h>
 #include <eel/eel-string.h>
-#include <eel/eel-xml-extensions.h>
 #include <libxml/parser.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
@@ -321,7 +319,7 @@ nautilus_navigation_window_update_spatial_menu_item (NautilusNavigationWindow *w
 	action = gtk_action_group_get_action (window->details->navigation_action_group,
 					      NAUTILUS_ACTION_FOLDER_WINDOW);
 	gtk_action_set_visible (action,
-				!eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ALWAYS_USE_BROWSER));
+				!g_settings_get_boolean (nautilus_preferences, NAUTILUS_PREFERENCES_ALWAYS_USE_BROWSER));
 }
 
 static void
@@ -398,7 +396,7 @@ connect_proxy_cb (GtkActionGroup *action_group,
 	if (!GTK_IS_MENU_ITEM (proxy))
 		return;
 
-	label = GTK_LABEL (GTK_BIN (proxy)->child);
+	label = GTK_LABEL (gtk_bin_get_child (GTK_BIN (proxy)));
 
 	gtk_label_set_use_underline (label, FALSE);
 	gtk_label_set_ellipsize (label, PANGO_ELLIPSIZE_END);
@@ -585,37 +583,9 @@ action_new_tab_callback (GtkAction *action,
 			 gpointer user_data)
 {
 	NautilusWindow *window;
-	NautilusWindowSlot *current_slot;
-	NautilusWindowSlot *new_slot;
-	NautilusWindowOpenFlags flags;
-	GFile *location;
-	int new_slot_position;
-	char *scheme;
 
 	window = NAUTILUS_WINDOW (user_data);
-	current_slot = window->details->active_pane->active_slot;
-	location = nautilus_window_slot_get_location (current_slot);
-
-	if (location != NULL) {
-		flags = 0;
-
-		new_slot_position = eel_preferences_get_enum (NAUTILUS_PREFERENCES_NEW_TAB_POSITION);
-		if (new_slot_position == NAUTILUS_NEW_TAB_POSITION_END) {
-			flags = NAUTILUS_WINDOW_OPEN_SLOT_APPEND;
-		}
-
-		scheme = g_file_get_uri_scheme (location);
-		if (!strcmp (scheme, "x-nautilus-search")) {
-			g_object_unref (location);
-			location = g_file_new_for_path (g_get_home_dir ());
-		}
-		g_free (scheme);
-
-		new_slot = nautilus_window_open_slot (current_slot->pane, flags);
-		nautilus_window_set_active_slot (window, new_slot);
-		nautilus_window_slot_go_to (new_slot, location, FALSE);
-		g_object_unref (location);
-	}
+	nautilus_window_new_tab (window);
 }
 
 static void
@@ -802,7 +772,7 @@ static const GtkActionEntry navigation_entries[] = {
                                  NULL, N_("Open a folder window for the displayed location"),
                                  G_CALLBACK (action_folder_window_callback) },
   /* name, stock id, label */  { "Close All Windows", NULL, N_("Close _All Windows"),
-                                 "<control><shift>W", N_("Close all Navigation windows"),
+                                 "<control>Q", N_("Close all Navigation windows"),
                                  G_CALLBACK (action_close_all_windows_callback) },
   /* name, stock id, label */  { "Go to Location", NULL, N_("_Location..."),
                                  "<control>L", N_("Specify a location to open"),
@@ -810,10 +780,10 @@ static const GtkActionEntry navigation_entries[] = {
   /* name, stock id, label */  { "Clear History", NULL, N_("Clea_r History"),
                                  NULL, N_("Clear contents of Go menu and Back/Forward lists"),
                                  G_CALLBACK (action_clear_history_callback) },
-  /* name, stock id, label */  { "SplitViewNextPane", NULL, N_("Switch to other pane"),
+  /* name, stock id, label */  { "SplitViewNextPane", NULL, N_("S_witch to Other Pane"),
 				 "F6", N_("Move focus to the other pane in a split view window"),
 				 G_CALLBACK (action_split_view_switch_next_pane_callback) },
-  /* name, stock id, label */  { "SplitViewSameLocation", NULL, N_("Same location as other pane"),
+  /* name, stock id, label */  { "SplitViewSameLocation", NULL, N_("Sa_me Location as Other Pane"),
 				 NULL, N_("Go to the same location as in the extra pane"),
 				 G_CALLBACK (action_split_view_same_location_callback) },
   /* name, stock id, label */  { "Add Bookmark", GTK_STOCK_ADD, N_("_Add Bookmark"),
@@ -834,7 +804,7 @@ static const GtkActionEntry navigation_entries[] = {
 	{ "TabsMoveRight", NULL, N_("Move Tab _Right"), "<shift><control>Page_Down",
 	  N_("Move current tab to right"),
 	  G_CALLBACK (action_tabs_move_right_callback) },
-	{ "ShowSearch", NULL, N_("Show search"), "<control>f",
+	{ "ShowSearch", NULL, N_("S_how Search"), "<control>f",
 	  N_("Show search"),
 	  G_CALLBACK (action_show_search_callback) }
 };
@@ -867,7 +837,7 @@ static const GtkToggleActionEntry navigation_toggle_entries[] = {
                              G_CALLBACK (action_show_hide_search_callback),
   /* is_active */            FALSE },
   /* name, stock id */     { NAUTILUS_ACTION_SHOW_HIDE_EXTRA_PANE, NULL,
-  /* label, accelerator */   N_("Extra Pane"), "F3",
+  /* label, accelerator */   N_("E_xtra Pane"), "F3",
   /* tooltip */              N_("Open an extra folder view side-by-side"),
                              G_CALLBACK (action_split_view_callback),
   /* is_active */            FALSE },
