@@ -23,7 +23,6 @@
 #include <eel/eel-debug.h>
 #include <eel/eel-gtk-extensions.h>
 #include <eel/eel-glib-extensions.h>
-#include <eel/eel-preferences.h>
 #include <eel/eel-string.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
@@ -304,10 +303,9 @@ nautilus_path_bar_init (NautilusPathBar *path_bar)
 	path_bar->root_path = g_file_new_for_path ("/");
 	desktop_is_home = g_file_equal (path_bar->home_path, path_bar->desktop_path);
 
-	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_DESKTOP_IS_HOME_DIR,
-						  desktop_location_changed_callback,
-						  path_bar,
-						  G_OBJECT (path_bar));
+	g_signal_connect_swapped (nautilus_preferences, "changed::" NAUTILUS_PREFERENCES_DESKTOP_IS_HOME_DIR,
+				  G_CALLBACK(desktop_location_changed_callback),
+				  path_bar);
 
         g_signal_connect_swapped (path_bar->up_slider_button, "clicked", G_CALLBACK (nautilus_path_bar_scroll_up), path_bar);
         g_signal_connect_swapped (path_bar->down_slider_button, "clicked", G_CALLBACK (nautilus_path_bar_scroll_down), path_bar);
@@ -427,6 +425,9 @@ nautilus_path_bar_finalize (GObject *object)
 
 	g_signal_handlers_disconnect_by_func (nautilus_trash_monitor_get (),
 					      trash_state_changed_cb, path_bar);
+	g_signal_handlers_disconnect_by_func (nautilus_preferences,
+					      desktop_location_changed_callback,
+					      path_bar);
 
         G_OBJECT_CLASS (nautilus_path_bar_parent_class)->finalize (object);
 }
@@ -1509,7 +1510,7 @@ button_data_file_changed (NautilusFile *file,
 			  ButtonData *button_data)
 {
 	GFile *location, *current_location, *parent, *button_parent;
-	ButtonData *current_button_data;
+	ButtonData *current_button_data, *temp_button_data;
 	char *display_name;
 	NautilusPathBar *path_bar;
 	gboolean renamed, child;
@@ -1569,27 +1570,20 @@ button_data_file_changed (NautilusFile *file,
 			return;
 		}
 	} else if (nautilus_file_is_gone (file)) {
-		gint idx, position;
+		/* remove this and the following buttons */
+		GList *children;
 
-		/* if the current or a parent location are gone, don't do anything, as the view
-		 * will get the event too and call us back.
-		 */
-		current_location = nautilus_file_get_location (current_button_data->file);
+		children = g_list_find (path_bar->button_list, button_data);
 
-		if (g_file_has_prefix (location, current_location)) {
-			/* remove this and the following buttons */
-			position = g_list_position (path_bar->button_list,
-						    g_list_find (path_bar->button_list, button_data));
+		while (children != NULL) {
+			temp_button_data = children->data;
 
-			if (position != -1) {
-				for (idx = 0; idx <= position; idx++) {
-					gtk_container_remove (GTK_CONTAINER (path_bar),
-							      BUTTON_DATA (path_bar->button_list->data)->button);
-				}
-			}
+			/* children are in reverse order */
+			children = children->prev;
+
+			gtk_container_remove (GTK_CONTAINER (path_bar), temp_button_data->button);
 		}
 
-		g_object_unref (current_location);
 		g_object_unref (location);
 		return;
 	}
