@@ -142,9 +142,9 @@ set_displayed_location (NautilusWindowSlot *slot, GFile *location)
 			g_object_unref (slot->last_location_bookmark);
                 }
 		slot->last_location_bookmark = slot->current_location_bookmark;
-		name = g_file_get_uri (location);
+		name = g_file_get_basename (location);
 		slot->current_location_bookmark = (location == NULL) ? NULL
-                        : nautilus_bookmark_new (location, name, TRUE, NULL);
+                        : nautilus_bookmark_new (location, name, FALSE, NULL);
 		g_free (name);
         }
 }
@@ -516,7 +516,7 @@ nautilus_window_slot_open_location_full (NautilusWindowSlot *slot,
 	old_location = nautilus_window_slot_get_location (slot);
 	switch (mode) {
         case NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE :
-		if (eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ALWAYS_USE_BROWSER)) {
+		if (g_settings_get_boolean (nautilus_preferences, NAUTILUS_PREFERENCES_ALWAYS_USE_BROWSER)) {
 			target_window = window;
 			if (NAUTILUS_IS_SPATIAL_WINDOW (window)) {
 				if (!NAUTILUS_SPATIAL_WINDOW (window)->affect_spatial_window_on_next_location_change) {
@@ -584,7 +584,7 @@ nautilus_window_slot_open_location_full (NautilusWindowSlot *slot,
 
 		slot_flags = 0;
 
-		new_slot_position = eel_preferences_get_enum (NAUTILUS_PREFERENCES_NEW_TAB_POSITION);
+		new_slot_position = g_settings_get_enum (nautilus_preferences, NAUTILUS_PREFERENCES_NEW_TAB_POSITION);
 		if (new_slot_position == NAUTILUS_NEW_TAB_POSITION_END) {
 			slot_flags = NAUTILUS_WINDOW_OPEN_SLOT_APPEND;
 		}
@@ -1062,6 +1062,7 @@ got_file_info_for_view_selection_callback (NautilusFile *file,
 		slot->tried_mount = TRUE;
 
 		mount_op = gtk_mount_operation_new (GTK_WINDOW (window));
+		g_mount_operation_set_password_save (mount_op, G_PASSWORD_SAVE_FOR_SESSION);
 		location = nautilus_file_get_location (file);
 		data = g_new0 (MountNotMountedData, 1);
 		data->cancellable = g_cancellable_new ();
@@ -1399,7 +1400,7 @@ location_has_really_changed (NautilusWindowSlot *slot)
 	if (slot->new_content_view != NULL) {
 		widget = nautilus_view_get_widget (slot->new_content_view);
 		/* Switch to the new content view. */
-		if (widget->parent == NULL) {
+		if (gtk_widget_get_parent (widget) == NULL) {
 			if (slot->content_view != NULL) {
 				nautilus_window_slot_disconnect_content_view (slot, slot->content_view);
 			}
@@ -1925,6 +1926,7 @@ display_view_selection_failure (NautilusWindow *window, NautilusFile *file,
 			detail_message = g_strdup (_("Check that the spelling is correct and that your proxy settings are correct."));
 			break;
 		case G_IO_ERROR_CANCELLED:
+		case G_IO_ERROR_FAILED_HANDLED:
 			g_free (uri_for_display);
 			return;
 			
