@@ -54,8 +54,8 @@ struct NautilusWindowDetails
         GtkActionGroup *extensions_menu_action_group;
 
         GtkActionGroup *bookmarks_action_group;
-        guint refresh_bookmarks_menu_idle_id;
         guint bookmarks_merge_id;
+        NautilusBookmarkList *bookmark_list;
 
 	NautilusWindowShowHiddenFilesMode show_hidden_files_mode;
 
@@ -81,6 +81,11 @@ struct NautilusWindowDetails
          */
         GList *panes;
         NautilusWindowPane *active_pane;
+
+	/* So we can tell which window initiated
+	 * an unmount operation.
+	 */
+	gboolean initiated_unmount;
 };
 
 struct _NautilusNavigationWindowDetails {
@@ -105,9 +110,9 @@ struct _NautilusNavigationWindowDetails {
         guint extensions_toolbar_merge_id;
         GtkActionGroup *extensions_toolbar_action_group;
 
-	/* Throbber */
-        gboolean    throbber_active;
-        GtkWidget  *throbber;
+        /* spinner */
+        gboolean    spinner_active;
+        GtkWidget  *spinner;
 
         /* focus widget before the location bar has been shown temporarily */
         GtkWidget *last_focus_widget;
@@ -137,20 +142,20 @@ struct _NautilusNavigationWindowDetails {
 #define NAUTILUS_COMMAND_ZOOM_NORMAL			"/commands/Zoom Normal"
 
 /* window geometry */
-/* These are very small, and a Nautilus window at this tiny size is *almost*
+/* Min values are very small, and a Nautilus window at this tiny size is *almost*
  * completely unusable. However, if all the extra bits (sidebar, location bar, etc)
  * are turned off, you can see an icon or two at this size. See bug 5946.
  */
-#define NAUTILUS_WINDOW_MIN_WIDTH			200
-#define NAUTILUS_WINDOW_MIN_HEIGHT			200
 
 #define NAUTILUS_SPATIAL_WINDOW_MIN_WIDTH			100
 #define NAUTILUS_SPATIAL_WINDOW_MIN_HEIGHT			100
 #define NAUTILUS_SPATIAL_WINDOW_DEFAULT_WIDTH			500
 #define NAUTILUS_SPATIAL_WINDOW_DEFAULT_HEIGHT			300
 
-#define NAUTILUS_NAVIGATION_WINDOW_DEFAULT_WIDTH        800
-#define NAUTILUS_NAVIGATION_WINDOW_DEFAULT_HEIGHT       550
+#define NAUTILUS_NAVIGATION_WINDOW_MIN_WIDTH			200
+#define NAUTILUS_NAVIGATION_WINDOW_MIN_HEIGHT			200
+#define NAUTILUS_NAVIGATION_WINDOW_DEFAULT_WIDTH		800
+#define NAUTILUS_NAVIGATION_WINDOW_DEFAULT_HEIGHT		550
 
 typedef void (*NautilusBookmarkFailedCallback) (NautilusWindow *window,
                                                 NautilusBookmark *bookmark);
@@ -172,9 +177,6 @@ void               nautilus_menus_append_bookmark_to_menu                (Nautil
                                                                           guint              merge_id,
                                                                           GCallback          refresh_callback,
                                                                           NautilusBookmarkFailedCallback failed_callback);
-#ifdef NEW_UI_COMPLETE
-void               nautilus_window_go_up                                 (NautilusWindow    *window);
-#endif
 void               nautilus_window_update_find_menu_item                 (NautilusWindow    *window);
 void               nautilus_window_zoom_in                               (NautilusWindow    *window);
 void               nautilus_window_zoom_out                              (NautilusWindow    *window);
@@ -207,7 +209,6 @@ gboolean           nautilus_add_to_history_list_no_notify                (GFile
 									  GIcon            *icon);
 GList *            nautilus_get_history_list                             (void);
 void               nautilus_window_bookmarks_preference_changed_callback (gpointer           user_data);
-void               nautilus_window_constructed                           (NautilusWindow    *window);
 
 
 /* sync window GUI with current slot. Used when changing slots,
@@ -232,10 +233,10 @@ void               nautilus_navigation_window_remove_go_menu_callback    (Nautil
 void               nautilus_navigation_window_remove_go_menu_items       (NautilusNavigationWindow    *window);
 
 /* Navigation window toolbar */
-void               nautilus_navigation_window_activate_throbber                     (NautilusNavigationWindow    *window);
+void               nautilus_navigation_window_activate_spinner                     (NautilusNavigationWindow    *window);
 void               nautilus_navigation_window_initialize_toolbars                   (NautilusNavigationWindow    *window);
 void               nautilus_navigation_window_load_extension_toolbar_items          (NautilusNavigationWindow    *window);
-void               nautilus_navigation_window_set_throbber_active                   (NautilusNavigationWindow    *window, 
+void               nautilus_navigation_window_set_spinner_active                   (NautilusNavigationWindow    *window, 
                                                                                      gboolean                     active);
 void               nautilus_navigation_window_go_back                               (NautilusNavigationWindow    *window);
 void               nautilus_navigation_window_go_forward                            (NautilusNavigationWindow    *window);
